// RROS2CLI - Road Runner Login COMMAND LINE INTERFACE
//
//

#include <rros2protocol.h>

#include <stdio.h>
#include <stdlib.h>

#include <io.h>


const char AuthName[4][8]={"Unknown","Toshiba","HP     ","Digital"};
int sock, sunlen;
struct sockaddr_un sun;


int ShowError(RROS2_MESSAGE *inmsg)
{
	int rc;
	char *string;

	if (!inmsg)
		return -1;

	if (RROS2_GetInt(inmsg,&rc)<0 || RROS2_GetString(inmsg,&string)<0)
		return -2;

	printf("ERROR: %s (rc=%d)\n",string,rc);
	return 0;
}

int DoDetectAuth()
{
	int rc, type, detected;
	char *message;
	RROS2_MESSAGE *outmsg,*inmsg;

	outmsg=RROS2_NewMessage();
	outmsg->Request=RROS2_DETECT_AUTH;

	if ((inmsg=RROS2_Transcieve(sock,outmsg))==NULL)
	{
		delete outmsg;
		return -1;
	}
	delete outmsg;

	if (inmsg->Response==RROS2_FAILED)
	{
		printf("Not OK: %d\n",inmsg->Response);
		ShowError(inmsg);
		delete inmsg;
		return -1;
	}
	detected=0;
	if (inmsg->Space)
	{
		while (RROS2_GetInt(inmsg,&type)==0 && type)
		{
			detected=type;
			RROS2_GetInt(inmsg,&rc);
			RROS2_GetString(inmsg,&message);
			if (type>3)
				printf("%d\t%d\t%s\n",type,rc,message);
			else
				printf("%s\t%d\t%s\n",AuthName[type],rc,message);fflush(stdout);
		}
		if (detected>3)
			printf("An Unknown");
		else
			printf("%s",AuthName[detected]);
		printf(" Authentication system detected.\n");
	}

	delete inmsg;
	return 0;
}

int DoSetAuth(char *authname)
{
	int rc, count;
	RROS2_MESSAGE *outmsg,*inmsg;

	if (!authname)
		return -1;

	outmsg=RROS2_NewMessage();
	outmsg->Request=RROS2_SET_AUTH;

	if (strcmp(authname,"toshiba")==0)
		RROS2_AddInt(outmsg,RROS2_TOSHIBA);
	else if (strcmp(authname,"hp")==0)
		RROS2_AddInt(outmsg,RROS2_HP);
	else if (strcmp(authname,"digital")==0)
		RROS2_AddInt(outmsg,RROS2_DIGITAL);
	else
	{
		printf("Unknown auth: %s\n",authname);
		return -2;
	}

	if ((inmsg=RROS2_Transcieve(sock,outmsg))==NULL)
	{
		delete outmsg;
		return -1;
	}
	delete outmsg;

	if (inmsg->Response!=RROS2_OK)
	{
		printf("Not OK: %d\n",inmsg->Response);
		ShowError(inmsg);
		delete inmsg;
		return -1;
	}
	printf("Authentication was set\n");
	delete inmsg;
	return 0;
}

int DoLoginState()
{
	int rc, count;
	RROS2_MESSAGE *outmsg,*inmsg;

	outmsg=RROS2_NewMessage();
	outmsg->Request=RROS2_LOGIN_STATE;

	if ((inmsg=RROS2_Transcieve(sock,outmsg))==NULL)
	{
		delete outmsg;
		return -1;
	}
	delete outmsg;

	if (inmsg->Response!=RROS2_OK)
	{
		printf("Not OK: %d\n",inmsg->Response);
		ShowError(inmsg);
		delete inmsg;
		return -1;
	}
	delete inmsg;
	printf("User is logged in\n");
	return 0;
}

int DoLogin(char *username, char *password)
{
	int rc, count;
	RROS2_MESSAGE *outmsg,*inmsg;

	outmsg=RROS2_NewMessage();
	outmsg->Request=RROS2_LOGIN;

	RROS2_AddString(outmsg,username);
	RROS2_AddString(outmsg,password);

	if ((inmsg=RROS2_Transcieve(sock,outmsg))==NULL)
	{
		delete outmsg;
		return -1;
	}
	delete outmsg;

	if (inmsg->Response!=RROS2_OK)
	{
		printf("Not OK: %d\n",inmsg->Response);
		ShowError(inmsg);
		delete inmsg;
		return -1;
	}
	delete inmsg;
	printf("Login successful\n");
	return 0;
}		

int DoLogout()
{
	RROS2_MESSAGE *outmsg,*inmsg;

	outmsg=RROS2_NewMessage();
	outmsg->Request=RROS2_LOGOUT;

	if ((inmsg=RROS2_Transcieve(sock,outmsg))==NULL)
	{
		delete outmsg;
		return -1;
	}
	delete outmsg;

	if (inmsg->Response!=RROS2_OK)
	{
		printf("Not OK: %d\n",inmsg->Response);
		ShowError(inmsg);
		delete inmsg;
		return -1;
	}
	delete inmsg;
	printf("Logout successful\n");
	return 0;
}

int DoListVars()
{
	char *name, *value;
	RROS2_MESSAGE *outmsg,*inmsg;

	outmsg=RROS2_NewMessage();
	outmsg->Request=RROS2_LIST_VARS;

	if ((inmsg=RROS2_Transcieve(sock,outmsg))==NULL)
	{
		delete outmsg;
		printf("Trancieve failed\n");
		return -1;
	}
	delete outmsg;

	if (inmsg->Response!=RROS2_OK)
	{
		printf("Not OK: %d\n",inmsg->Response);
		ShowError(inmsg);
		delete inmsg;
		return -1;
	}
	while (RROS2_GetString(inmsg,&name)==0 && RROS2_GetString(inmsg,&value)==0 && *name)
	{
		printf("\t%s = %s\n",name,value);fflush(stdout);
	}

	delete inmsg;
	return 0;
}

int DoSetVar(char *name, char *value)
{
	int rc, count;
	RROS2_MESSAGE *outmsg,*inmsg;

	outmsg=RROS2_NewMessage();
	outmsg->Request=RROS2_SET_VAR;

	RROS2_AddString(outmsg,name);
	if (!value)
		RROS2_AddString(outmsg,"");
	else
		RROS2_AddString(outmsg,value);

	if ((inmsg=RROS2_Transcieve(sock,outmsg))==NULL)
	{
		delete outmsg;
		return -1;
	}
	delete outmsg;

	if (inmsg->Response!=RROS2_OK)
	{
		printf("Not OK: %d\n",inmsg->Response);
		ShowError(inmsg);
		delete inmsg;
		return -1;
	}

	delete inmsg;
	printf("Variable was set\n");
	return 0;
}

int DoGetVar(char *name)
{
	char *value;
	RROS2_MESSAGE *outmsg,*inmsg;

	outmsg=RROS2_NewMessage();
	outmsg->Request=RROS2_GET_VAR;

	RROS2_AddString(outmsg,name);

	if ((inmsg=RROS2_Transcieve(sock,outmsg))==NULL)
	{
		delete outmsg;
		return -1;
	}
	delete outmsg;

	if (inmsg->Response!=RROS2_OK)
	{
		printf("Not OK: %d\n",inmsg->Response);
		ShowError(inmsg);
		delete inmsg;
		return -1;
	}
	if (RROS2_GetString(inmsg,&value)<0)
		printf("Value not in payload\n");
	else
		printf("%s = %s\n",name,value);

	delete inmsg;
	return 0;
}

int DoGetError()
{
	char *value;
	RROS2_MESSAGE *outmsg,*inmsg;

	outmsg=RROS2_NewMessage();
	outmsg->Request=RROS2_GET_ERROR;

	if ((inmsg=RROS2_Transcieve(sock,outmsg))==NULL)
	{
		delete outmsg;
		return -1;
	}
	delete outmsg;

	if (inmsg->Response!=RROS2_OK)
	{
		printf("Not OK: %d\n",inmsg->Response);
		ShowError(inmsg);
		delete inmsg;
		return -1;
	}
	if (RROS2_GetString(inmsg,&value)<0)
		printf("Error Message not in payload\n");
	else
		printf("Error Message = %s\n",value);

	delete inmsg;
	return 0;
}

int DoShutdown()
{
	RROS2_MESSAGE *outmsg,*inmsg;

	outmsg=RROS2_NewMessage();
	outmsg->Request=RROS2_SHUTDOWN;

	if ((inmsg=RROS2_Transcieve(sock,outmsg))==NULL)
	{
		delete outmsg;
		return -1;
	}
	delete outmsg;

	if (inmsg->Response!=RROS2_OK)
	{
		printf("Not OK: %d\n",inmsg->Response);
		ShowError(inmsg);
		delete inmsg;
		return -1;
	}
	delete inmsg;
	printf("Shutdown command accepted\n");
	return 0;
}

Usage()
{
	printf("RROS2cli Command Line Usage:\n");
	printf("********************************************************************************");
	printf("detect                      - Autodetect Authentication System.\n");
	printf("setauth: <n>                - Manually Set Authentication System.\n");
	printf("                              n = 1 for Toshiba\n");
	printf("                              2 for HP\n");
	printf("                              3 for HP\n");
	printf("state                       - Determine the true Login State of the User.\n");
	printf("login <username> <password> - Login to Road Runner.\n");
	printf("logout                      - Logout of Road Runner.\n");
	printf("list                        - Display all Internal Variables.\n");
	printf("set <name> [value]          - Set/Delete an Internal Variable.\n");
	printf("                              If you do NOT specify a value the named\n");
	printf("                              variable will be deleted.\n");
	printf("get <name>                  - Get an Internal Variable.\n");
	printf("\n\n");
}

int main(int argc, char **argv)
{
	struct sockaddr_un sun;

	if (argc<2)
	{
		Usage();
		return -1;
	}

	if ((sock=socket(AF_UNIX,SOCK_DGRAM,0))==-1)
	{
		printf("Unable to allocate socket\n");
		return -1;
	}

	sun.sun_family=AF_UNIX;
	strcpy(sun.sun_path,"");
	if (bind(sock,(struct sockaddr *)&sun,sizeof(sun))<0)
	{
		printf("Could Not Bind Socket (errno=%d)\n",errno);
		return -1;
	}

	if (strcmp(argv[1],"detect")==0)
		DoDetectAuth();
	else if (strcmp(argv[1],"setauth")==0)
	{
		if (argc<3)
		{
			printf("setauth requires a name\n");
			exit(-1);
		}
		DoSetAuth(argv[2]);
	}
	else if (strcmp(argv[1],"state")==0)
		DoLoginState();
	else if (strcmp(argv[1],"login")==0)
	{
		if (argc<4)
		{
			printf("login requires a name and password\n");
			exit(-1);
		}
		DoLogin(argv[2],argv[3]);
	}
	else if (strcmp(argv[1],"logout")==0)
		DoLogout();
	else if (strcmp(argv[1],"list")==0)
		DoListVars();
	else if (strcmp(argv[1],"set")==0)
	{
		if (argc<3)
		{
			printf("set requires at least a name\n");
			exit(-1);
		}
		DoSetVar(argv[2], argv[3]);
	}
	else if (strcmp(argv[1],"get")==0)
	{
		if (argc<3)
		{
			printf("get requires a name\n");
			exit(-1);
		}
		DoGetVar(argv[2]);
	}
	else if (strcmp(argv[1],"error")==0)
		DoGetError();
	else if (strcmp(argv[1],"shutdown")==0)
		DoShutdown();
	else
	{
		printf("Unknown command: %s\n",argv[1]);
		Usage();
		return -1;
	}
	close(sock);
}